//--------------------------------------------------------------------------------------------------
//
// Title       : mips_registers
// Design      : Base Components
// Author      : Steven
// Company     : UW CSE
//
//-------------------------------------------------------------------------------------------------
//
// File        : mips_registers.v
// Generated   : Sun Apr 9 20:45:40 2006
// From        : interface description file
// By          : Itf2Vhdl ver. 1.20
//
//-------------------------------------------------------------------------------------------------
//
// Description : 
//
//-------------------------------------------------------------------------------------------------
`timescale 1ns / 1ps


// Arbitrary width register
module register(D, CLK, Q) /* synthesis syn_black_box */ ;
	parameter EDGE=1'b1; // 1 = posedge, 0 = negedge
	parameter WIDTH=32;	
	parameter INIT=32'h0;
	input wire [WIDTH-1:0] D;
	input wire CLK;
	output wire [WIDTH-1:0] Q;	
	
	// synopsys translate_off
	
`ifdef SYNTHESIS	
	genvar i; // loop variable for generate statement
	generate 						
		for (i = 0; i < WIDTH; i = i + 1) 
			begin:val
				if(EDGE == 1'b1) // use posedge trigger FF
					begin
	
						`library("ff","ovi_unisim")

						FD #(.INIT(INIT[i])) ff (.D(D[i]),.C(CLK),.Q(Q[i]));
					end
				else		    // use negedge trigger FF
					begin

						`library("ff","ovi_unisim")
	
						FD_1 #(.INIT(INIT[i])) ff (.D(D[i]),.C(CLK),.Q(Q[i]));
					end
			end
	endgenerate	
`else
	reg [WIDTH-1:0] FF;
	initial begin
		FF[WIDTH-1:0] = INIT[WIDTH-1:0];
	end
	always @(posedge CLK) begin
		FF <= D;
	end
	assign Q = FF;
`endif 
// synopsys translate_on
endmodule

// Arbitrary width register with reset
module register_r(D,CLK,RESET,Q) /* synthesis syn_black_box */ ;
	parameter EDGE=1'b1; // 1 = posedge, 0 = negedge
	parameter SYNC_R=1'b1; // 1 = synchronous reset, 0 = async reset
	parameter WIDTH=32;	
	parameter INIT=0;
	input wire [WIDTH-1:0] D;
	input wire RESET,CLK;
	output wire [WIDTH-1:0] Q; 
	
	// synopsys translate_off
	
`ifdef SYNTHESIS	
	genvar i; // loop variable for generate statement
	generate 						
		for (i = 0; i < WIDTH; i = i + 1) 
			begin:val
				if(EDGE == 1'b1) // use posedge trigger FF 	
					begin
						if(SYNC_R == 1'b1) // use synchronous reset	
							begin

								`library("ff","ovi_unisim")

								FDR #(.INIT(INIT[i])) ff (.D(D[i]),.C(CLK),.Q(Q[i]),.R(RESET));
							end
						else 
							begin

								`library("ff","ovi_unisim")

								FDC #(.INIT(INIT[i])) ff (.D(D[i]),.C(CLK),.Q(Q[i]),.CLR(RESET));
							end
					end
				else		    // use negedge trigger FF  
					begin
						if(SYNC_R == 1'b1) // use synchronous reset
							begin

								`library("ff","ovi_unisim")

								FDR_1 #(.INIT(INIT[i])) ff (.D(D[i]),.C(CLK),.Q(Q[i]),.R(RESET));
							end
						else 
							begin
	
								`library("ff","ovi_unisim")
	
								FDC_1 #(.INIT(INIT[i])) ff (.D(D[i]),.C(CLK),.Q(Q[i]),.CLR(RESET));
							end
					end
			end
	endgenerate	
`else
	reg [WIDTH-1:0] FF;
	initial begin
		FF[WIDTH-1:0] = INIT[WIDTH-1:0];
	end
	always @(posedge CLK) begin
		FF <= (RESET)? 0 : D;
	end
	assign Q = FF;
`endif
// synopsys translate_on	
endmodule

// Arbitrary width register with reset, clock enable
module register_re(D,CLK,RESET,LD,Q) /* synthesis syn_black_box */ ;
	parameter EDGE=1'b1; // 1 = posedge, 0 = negedge
	parameter SYNC_R=1'b1; // 1 = synchronous reset, 0 = async reset
	parameter WIDTH=32;	
	parameter INIT=0;
	input wire [WIDTH-1:0] D;
	input wire RESET,CLK,LD;
	output wire [WIDTH-1:0] Q;
	
	// synopsys translate_off
	
`ifdef SYNTHESIS

	`library("ff","ovi_unisim")

	genvar i; // loop variable for generate statement
	generate 						
		for (i = 0; i < WIDTH; i = i + 1) 
			begin:val
				if(EDGE == 1'b1)
					begin // use posedge trigger FF 
						if(SYNC_R == 1'b1) // use synchronous reset
							begin
	
								`library("ff","ovi_unisim")
							
								FDRE #(.INIT(INIT[i])) ff (.D(D[i]),.C(CLK),.Q(Q[i]),.R(RESET),.CE(LD));
							end 
						else 
							begin   
							
								`library("ff","ovi_unisim")
							
								FDCE #(.INIT(INIT[i])) ff (.D(D[i]),.C(CLK),.Q(Q[i]),.CLR(RESET),.CE(LD));
							end
					end
				else		    // use negedge trigger FF
					begin
						if(SYNC_R == 1'b1) // use synchronous reset
							begin
							
								`library("ff","ovi_unisim")
							
								FDRE_1 #(.INIT(INIT[i])) ff (.D(D[i]),.C(CLK),.Q(Q[i]),.R(RESET),.CE(LD));
							end
						else
							begin
							
								`library("ff","ovi_unisim")
							
								FDCE_1 #(.INIT(INIT[i])) ff (.D(D[i]),.C(CLK),.Q(Q[i]),.CLR(RESET),.CE(LD));
							end
					end
			end
	endgenerate	
`else
	reg [WIDTH-1:0] FF;
	initial begin
		FF[WIDTH-1:0] = INIT[WIDTH-1:0];
	end
	always @(posedge CLK) begin
		FF <= (RESET)? 0 : (LD) ? D : FF;
	end
	assign Q = FF;
`endif			  

// synopsys translate_on
endmodule


