//--------------------------------------------------------------------------------------------------
//
// Title       : Memory Mapped Character Mode VGA Driver
// Design      : vga_charmode2
// Author      : Adrian Caulfield - adapted from 
// Company     : 
//
//-------------------------------------------------------------------------------------------------
//
// File        : VGA_Controller.v
// Generated   : Sat Apr 15 02:15:48 2006
//
//-------------------------------------------------------------------------------------------------
//
// Description : Top Level is VGA_Output
// Inputs:		
// DataIn:		Lowest 8 or 4 bits, for character or color data, contain the data to be written to the ram
// AddressIn:	The address in either the Character RAM or Color RAM
// CharWE:		Write enable lines for the character ram - writes only on 2'b01 - store byte
// ColorWE:		Write enable lines for the color ram - writes only on 2'b01 - store byte
// VGAClock:	VGA Clock from ClockSystem
// CLK:			Processor Clock
// Reset:		Should be tied to soft reset
// WE:			Write Enable for Character RAM
// Outputs:
//			All outputs should be routed to simliarly named pinouts on the FPGA
//-------------------------------------------------------------------------------------------------

/* Colors:
	The colors are:		DOS Index:
	Black				0
	Navy				1
	Green				2
	Teal				3
	Maroon				4
	Purple				5
	Olive				6
	Silver				7
	Grey				8
	Blue				9
	Lime Green			10
	Cyan				11
	Red					12
	Magenta				13
	Yellow				14
	White				15
*/



`ifdef _VCP
`else
`define library(a,b)
`define synthesis
`endif
 

// ---------- Design Unit Header ---------- //
`timescale 1ps / 1ps



module VGA_Output (VGAClock,CLK,Reset,CharWE,ColorWE,DataIn,AddressIn,VGA_COMP_SYNCH,VGA_HSYNCH,VGA_OUT_BLANK_Z,VGA_OUT_PIXEL_CLOCK,VGA_VSYNCH,VGA_OUT_BLUE,VGA_OUT_GREEN,
VGA_OUT_RED) /* synthesis syn_black_box */;

// ------------ Port declarations --------- //
input VGAClock;
wire VGAClock; 
input CLK;
wire CLK;
input Reset;
wire Reset;
input wire [7:0] DataIn;
input wire [12:0] AddressIn;
input wire CharWE;
input wire ColorWE;

output wire VGA_COMP_SYNCH;
output wire VGA_HSYNCH;
output wire VGA_OUT_BLANK_Z;
output wire VGA_OUT_PIXEL_CLOCK;
output wire VGA_VSYNCH;
output wire [7:0] VGA_OUT_BLUE;
output wire [7:0] VGA_OUT_GREEN;
output wire [7:0] VGA_OUT_RED;

//synopsys translate_off

COLOR_CHAR_MODE_SVGA_CTRL U1
(
	.VGA_COMP_SYNCH_N(VGA_COMP_SYNCH),
	.VGA_HSYNCH_N(VGA_HSYNCH),
	.VGA_OUT_BLANK_N(VGA_OUT_BLANK_Z),
	.VGA_OUT_BLUE_P(VGA_OUT_BLUE),
	.VGA_OUT_GREEN_P(VGA_OUT_GREEN),
	.VGA_OUT_PIXEL_CLOCK_P(VGA_OUT_PIXEL_CLOCK),
	.VGA_OUT_RED_P(VGA_OUT_RED),
	.VGA_VSYNCH_N(VGA_VSYNCH),
	.char_write_addr(AddressIn[12:0]),
	.char_write_clock(CLK),
	.char_write_data(DataIn[7:0]),
	.char_write_enable(CharWE),
	.color_write_addr(AddressIn[12:0]),
	.color_write_clock(CLK),
	.color_write_data(DataIn[3:0]),
	.color_write_enable(ColorWE),
	.pixel_clock(VGAClock),
	.reset(Reset)
);

//synopsys translate_on

endmodule 

module COLOR_CHAR_MODE_SVGA_CTRL (
VGA_OUT_PIXEL_CLOCK_P,
VGA_COMP_SYNCH_N,
VGA_OUT_BLANK_N,
VGA_HSYNCH_N,
VGA_VSYNCH_N,
VGA_OUT_RED_P,
VGA_OUT_GREEN_P,
VGA_OUT_BLUE_P,

char_write_addr,
char_write_data,
char_write_enable,
char_write_clock,

color_write_addr,
color_write_data,
color_write_enable,
color_write_clock,

pixel_clock,
reset
) /* synthesis syn_black_box */ ;

output 			VGA_OUT_PIXEL_CLOCK_P;	// pixel clock for the video DAC
output			VGA_COMP_SYNCH_N;		// composite sync for the video DAC
output 			VGA_OUT_BLANK_N;		// composite blanking for the video DAC
output 			VGA_HSYNCH_N;			// horizontal sync for the VGA output connector
output			VGA_VSYNCH_N;			// vertical sync for the VGA output connector
output [7:0] 	VGA_OUT_RED_P;			// RED DAC data
output [7:0] 	VGA_OUT_GREEN_P;		// GREEN DAC data
output [7:0] 	VGA_OUT_BLUE_P;			// BLUE DAC data

input [12:0]	char_write_addr;		// address of character to be written into memory
input [7:0]		char_write_data;		// ASCII data of character to be written
input			char_write_enable;		// write enable
input			char_write_clock		/* synthesis syn_noclockbuf = 1 */; // write clock

input [12:0]	color_write_addr;		// address of the color data to be written into memory
input [3:0]		color_write_data;		// character color data
input			color_write_enable;		// write enable
input			color_write_clock		/* synthesis syn_noclockbuf = 1 */; // write clock

input	 		pixel_clock				/* synthesis syn_noclockbuf = 1 */; // pixel clock input 
input			reset;					// reset  

//synopsys translate_off

wire			VGA_OUT_PIXEL_CLOCK_P;
reg 			VGA_COMP_SYNCH_N;
reg				VGA_OUT_BLANK_N;
reg				VGA_HSYNCH_N;
reg				VGA_VSYNCH_N;
reg	[7:0]		VGA_OUT_RED_P; 
reg	[7:0]		VGA_OUT_GREEN_P; 
reg	[7:0]		VGA_OUT_BLUE_P; 

wire [7:0]		dac_red_data;
wire [7:0]		dac_green_data;
wire [7:0]		dac_blue_data;


wire 			h_synch_delay;
wire 			v_synch_delay;
wire 			comp_synch;
wire 			blank;
wire [2:0]		char_line_count;
wire [12:0]		char_mode_address;
wire [7:0]		char_mode_data;
wire [12:0]		char_address;
wire [2:0]		char_pixel;

wire 			pixel_clock;
wire 			reset;

wire [7:0] 	red_data;
wire [7:0] 	green_data;
wire [7:0] 	blue_data;
wire		pixel_data;
wire [3:0] 	aligned_color_data;
wire [3:0] 	color_ram_data;

wire		color_read_clock;
wire [3:0] 	color_read_data;
wire [12:0] color_read_addr;
wire		color_read_enable;

wire		color_write_clock;
wire [3:0]  color_write_data;
wire [12:0] color_write_addr;
wire		color_write_enable;


assign char_address [12:0] = char_mode_address[12:0];

assign color_read_clock = pixel_clock;
assign color_read_enable = 1'b1;
assign color_read_addr[12:0] = char_mode_address[12:0];

assign pixel_data = char_mode_data[0];


// make the external connections
// shut down the video output during  reset
always @ (posedge pixel_clock or posedge reset) begin
	if (reset) begin
		VGA_HSYNCH_N	 	<= 1'b1;
		VGA_VSYNCH_N 	 	<= 1'b1;
		VGA_COMP_SYNCH_N 	<= 1'b1;
		VGA_OUT_BLANK_N 	<= 1'b0;
		VGA_OUT_RED_P		<= 8'h00;
		VGA_OUT_GREEN_P		<= 8'h00;
		VGA_OUT_BLUE_P		<= 8'h00;
		end
	else begin
		VGA_HSYNCH_N	 	<= !h_synch_delay;
		VGA_VSYNCH_N 	 	<= !v_synch_delay;
		VGA_COMP_SYNCH_N 	<= !comp_synch;
		VGA_OUT_BLANK_N 	<= !blank;
		VGA_OUT_RED_P		<= dac_red_data;
		VGA_OUT_GREEN_P		<= dac_green_data;
		VGA_OUT_BLUE_P		<= dac_blue_data;
		end
	end

assign  VGA_OUT_PIXEL_CLOCK_P	= pixel_clock;

// instantiate the SVGA timing generator
SVGA_TIMING_GENERATION SVGA_TIMING_GENERATION(
pixel_clock,
reset,
h_synch_delay,
v_synch_delay,
comp_synch,
blank,
char_line_count,
char_mode_address,
char_pixel
);

// instantiate the character RAM and character generator ROM
CHARACTER_MODE CHARACTER_MODE(
char_write_addr,
char_write_data,
char_write_enable,
char_write_clock,
char_address, 
char_line_count, 
char_pixel,
pixel_clock,
char_mode_data,	
reset
);


// instantiate the color RAM
COLOR_RAM COLOR_RAM (
color_read_clock,
color_read_data, 
color_read_addr, 
color_read_enable,
color_write_data,
color_write_addr,
color_write_clock,
color_write_enable);

// instantiate the color pipeline (just delays the color data by two clock cycles
COLOR_PIPE COLOR_PIPE (
color_read_data, 
pixel_clock, 
reset, 
aligned_color_data); 

// instantiate the color look up table
CLUT CLUT (
aligned_color_data,
pixel_clock, 
reset, 
red_data,
green_data, 
blue_data);

// instantiate the DAC driver
DRIVE_DAC_DATA DRIVE_DAC_DATA(
red_data, //3 cc
green_data, 
blue_data, 
char_mode_data,
dac_red_data,
dac_green_data,
dac_blue_data
); 

//synopsys translate_on

endmodule //COLOR_CHAR_MODE_SVGA_CTRL


module CHAR_RAM (read_clk,read_data,read_addr,read_enable,write_data,write_addr,write_clk,write_enable) /* synthesis syn_black_box */ ;
input		read_clk;
output [7:0]read_data;
input [12:0]read_addr;
input		read_enable;

input [7:0] write_data;
input [12:0]write_addr;
input		write_clk;
input		write_enable;

//synopsys translate_off

wire 		read_clk;
wire [7:0] 	read_data;
wire [12:0] read_addr;
wire 		read_enable;

wire 		write_clk;
wire [7:0] 	write_data;
wire [12:0] write_addr;
wire 		write_enable;


`library("RAM_BITS_1_0","virtex2p")
`library("RAM_BITS_3_2","virtex2p")
`library("RAM_BITS_5_4","virtex2p")
`library("RAM_BITS_7_6","virtex2p")


// instantiate the RAMs

RAMB16_S2_S2 RAM_BITS_1_0 (
.DOA(),
.ADDRA(write_addr[12:0]),
.CLKA(write_clk),
.DIA(write_data[1:0]),
.ENA(write_enable),
.WEA(1'b1),
.SSRA(1'b0),

.DOB(read_data[1:0]),
.ADDRB(read_addr[12:0]),
.CLKB(read_clk),
.DIB(2'b00),
.ENB(read_enable),
.WEB(1'b0),
.SSRB(1'b0)
);

RAMB16_S2_S2 RAM_BITS_3_2 (
.DOA(),
.ADDRA(write_addr[12:0]),
.CLKA(write_clk),
.DIA(write_data[3:2]),
.ENA(write_enable),
.WEA(1'b1),
.SSRA(1'b0),

.DOB(read_data[3:2]),
.ADDRB(read_addr[12:0]),
.CLKB(read_clk),
.DIB(2'b00),
.ENB(read_enable),
.WEB(1'b0),
.SSRB(1'b0)
);

RAMB16_S2_S2 RAM_BITS_5_4 (
.DOA(),
.ADDRA(write_addr[12:0]),
.CLKA(write_clk),
.DIA(write_data[5:4]),
.ENA(write_enable),
.WEA(1'b1),
.SSRA(1'b0),

.DOB(read_data[5:4]),
.ADDRB(read_addr[12:0]),
.CLKB(read_clk),
.DIB(2'b00),
.ENB(read_enable),
.WEB(1'b0),
.SSRB(1'b0)
);

RAMB16_S2_S2 RAM_BITS_7_6 (
.DOA(),
.ADDRA(write_addr[12:0]),
.CLKA(write_clk),
.DIA(write_data[7:6]),
.ENA(write_enable),
.WEA(1'b1),
.SSRA(1'b0),

.DOB(read_data[7:6]),
.ADDRB(read_addr[12:0]),
.CLKB(read_clk),
.DIB(2'b00),
.ENB(read_enable),
.WEB(1'b0),
.SSRB(1'b0)
);



//synopsys translate_on
endmodule //char_ram

module CHARACTER_MODE(
// write side
char_write_addr,
char_write_data,
char_write_enable,
char_write_clock,

// read side
char_address,
char_line_count,
char_pixel,
pixel_clock,
char_mode_data,

// control
reset
) /* synthesis syn_black_box */ ;

input			pixel_clock;
input			reset;
input [2:0]  	char_line_count;// line number within 8 line block
input [12:0] 	char_address;	// character address "0" is upper left character
input [2:0]  	char_pixel;		// pixel position within 8 pixel block
output [7:0]	char_mode_data;
input [12:0]   	char_write_addr;
input [7:0] 	char_write_data;
input			char_write_enable;
input 			char_write_clock;

//synopsys translate_off

reg [7:0] 		char_mode_data;
reg 			latch_data;
reg 			latch_low_data;
reg 			shift_high;
reg 			shift_low;
reg [3:0] 		latched_low_char_data;
reg [7:0] 		latched_char_data;
wire [7:0] 		char_gen_rom_data;
reg 			char_pix;
wire [7:0] 		ascii_code;
wire [2:0] 		char_line_count;
wire [12:0] 	char_write_addr;
wire [7:0] 		char_write_data;
wire 			char_write_enable;
wire 			char_write_clock;
wire [10:0] 	chargen_rom_address;
wire [12:0] 	char_address;		// extra bit allows for 1024 x 768 expansion


 
// instantiate the CHARACTER MODE DISPLAY RAM

CHAR_RAM CHAR_RAM(
.read_clk(pixel_clock),
.read_data(ascii_code[7:0]), //+1 cc
.read_addr(char_address[12:0]),	//0 cc
.read_enable(1'b1),
.write_data(char_write_data[7:0]),
.write_addr(char_write_addr),
.write_clk(char_write_clock),
.write_enable(char_write_enable)
);


// instantiate the character generator ROM


CHAR_GEN_ROM CHAR_GEN_ROM(
.pixel_clock(pixel_clock),
.address(chargen_rom_address), //1cc
.data(char_gen_rom_data[7:0]) //+1cc = 2cc
);
									 //1 cc			  //0cc
assign chargen_rom_address = {ascii_code[7:0],char_line_count[2:0]}; //1cc


// LATCH THE CHARTACTER DATA FROM THE CHAR GEN ROM AND CREATE A SERIAL CHAR DATA STREAM
always @ (posedge pixel_clock or posedge reset)begin
			if (reset) begin
				latch_data <= 1'b0;
				end
			else if (char_pixel == 3'b110) begin //if pixel column = 6
				latch_data <= 1'b1;	//read all 8 bits of the character data
				end
			else if (char_pixel == 3'b111) begin //if pixel column = 7
				latch_data <= 1'b0;
				end
			end

always @ (posedge pixel_clock or posedge reset)begin
			if (reset) begin
				latch_low_data <= 1'b0;
				end
			else if (char_pixel == 3'b010) begin //if pixel column = 2
				latch_low_data <= 1'b1; //copy lower 4 bits of latched data
				end
			else if (char_pixel == 3'b011) begin //if pixel column = 3
				latch_low_data <= 1'b0;
				end
			end

always @ (posedge pixel_clock or posedge reset)begin
			if (reset) begin
				shift_high <= 1'b1; 
				shift_low  <= 1'b0;
				end
			else if (char_pixel == 3'b011) begin //if pixel column = 3
				shift_high <= 1'b0;
				shift_low <= 1'b1;
				end
			else if (char_pixel == 3'b111) begin //if pixel column = 7
				shift_high <= 1'b1;
				shift_low <= 1'b0;
				end
			end


// serialize the CHARACTER MODE data
always @ (posedge pixel_clock or posedge reset) begin
	if (reset)
 		begin
			char_mode_data = 8'b0;
			latched_low_char_data = 4'h0;
			latched_char_data  = 8'h00;
		end

	else if (shift_high)
		begin
			char_mode_data = {8{latched_char_data [7]}};
			latched_char_data [7] = latched_char_data [6];
			latched_char_data [6] = latched_char_data [5];
			latched_char_data [5] = latched_char_data [4];
			latched_char_data [4] = latched_char_data [7];
				if(latch_low_data) begin
					latched_low_char_data [3:0] = latched_char_data [3:0];
					end
			end

	else if (shift_low)
		begin
			char_mode_data = {8{latched_low_char_data [3]}};
			latched_low_char_data [3] = latched_low_char_data [2];
			latched_low_char_data [2] = latched_low_char_data [1];
			latched_low_char_data [1] = latched_low_char_data [0];
			latched_low_char_data [0] = latched_low_char_data [3];
				if (latch_data) begin
					latched_char_data [7:0] = char_gen_rom_data[7:0];
				end
		end
	end
	
	//synopsys translate_on

endmodule //CHARACTER_MODE



module CLUT (aligned_color_data, pixel_clock, reset, red_data, green_data, blue_data) /* synthesis syn_black_box */ ;
input [3:0]  aligned_color_data;	// the color of the character
input		 pixel_clock;		// pixel clock
input		 reset;				// reset
output[7:0]  red_data;			// RED output data
output[7:0]  green_data;		// GREEN output data
output[7:0]  blue_data;			// BLUE output data

//synopsys translate_off

reg	[7:0]  red_data;		// RED output data
reg	[7:0]  green_data;	// GREEN output data
reg	[7:0]  blue_data;		// BLUE output data
reg	[7:0]  red;		
reg	[7:0]  green;	
reg	[7:0]  blue;

// create the color look up table
always @ (aligned_color_data) begin
case (aligned_color_data)
	4'b0000:begin		// pixel is BLACK
		red 	= 8'h00;
		green   = 8'h00;
		blue 	= 8'h00;
		end

	4'b0001:begin		// pixel is NAVY
		red 	= 8'h00;
		green 	= 8'h00;
		blue 	= 8'h80;
		end

	4'b0010:begin		// pixel is GREEN
		red 	= 8'h00;
		green 	= 8'h80;
		blue 	= 8'h00;
		end

	4'b0011:begin		// pixel is TEAL
		red 	= 8'h00;
		green 	= 8'h80;
		blue 	= 8'h80;
		end


	4'b0100:begin		// pixel is MAROON
		red 	= 8'h80;
		green 	= 8'h00;
		blue 	= 8'h00;
		end

	4'b0101:begin		// pixel is PURPLE
		red 	= 8'h80;
		green 	= 8'h00;
		blue 	= 8'h80;
		end

	4'b0110:begin		// pixel is OLIVE
		red 	= 8'h80;
		green 	= 8'h80;
		blue 	= 8'h00;
		end


	4'b0111:begin		// pixel is SILVER
		red 	= 8'hC0;
		green	= 8'hC0;
		blue 	= 8'hC0;
		end

	4'b1000:begin		// pixel is GREY
		red 	= 8'h80;
		green 	= 8'h80;
		blue 	= 8'h80;
		end

	4'b1001:begin		// pixel is BLUE
		red 	= 8'h00;
		green 	= 8'h00;
		blue 	= 8'hFF;
		end

	4'b1010:begin		// pixel is LIME GREEN
		red 	= 8'h00;
		green 	= 8'hFF;
		blue 	= 8'h00;
		end

	4'b1011:begin		// pixel is CYAN
		red 	= 8'h00;
		green 	= 8'hFF;
		blue 	= 8'hFF;
		end

	4'b1100:begin		// pixel is RED
		red 	= 8'hFF;
		green 	= 8'h00;
		blue 	= 8'h00;
		end

	4'b1101:begin		// pixel is CYAN
		red 	= 8'hFF;
		green	= 8'h00;
		blue 	= 8'hFF;
		end


	4'b1110:begin		// pixel is YELLOW
		red 	= 8'hFF;
		green 	= 8'hFF;
		blue 	= 8'h00;
		end

	4'b1111:begin		// pixel is WHITE
		red 	= 8'hFF;
		green 	= 8'hFF;
		blue 	= 8'hFF;
		end
endcase
end

always @ (posedge pixel_clock or posedge reset) begin
	if (reset) begin
		red_data 	<= 8'h00;
		green_data 	<= 8'h00;
		blue_data 	<= 8'h00;
		end
	else begin
		red_data 	<= red;
		green_data 	<= green;
		blue_data 	<= blue;
		end
end	

//synopsys translate_on

endmodule //CLUT


/*
Delay the Color Data so it matches with the correct characters
*/
module COLOR_PIPE (color_ram_data, pixel_clock, reset, aligned_color_data) /* synthesis syn_black_box */;
input	 [3:0] color_ram_data;		// the color of the character
input		 pixel_clock;		// pixel clock
input		 reset;			// reset
output[3:0]  aligned_color_data;	// time aligned color data

//synopsys translate_off

reg [3:0]	aligned_color_data;
reg [3:0]	delay0;

always @ (posedge pixel_clock or posedge reset) begin
	if (reset) begin
		aligned_color_data <= 4'h0;	
		delay0 <= 4'b0;
		end
else begin								 
		delay0 <= color_ram_data;
		aligned_color_data <= delay0;
	end
end

//synopsys translate_on

endmodule //COLOR_PIPE

module COLOR_RAM (color_read_clock,color_read_data,color_read_addr,color_read_enable,color_write_data,color_write_addr,color_write_clock,color_write_enable) /* synthesis syn_black_box */ ;
input		color_read_clock;
output [3:0]color_read_data;
input [12:0]color_read_addr;
input		color_read_enable;

input [3:0] color_write_data;
input [12:0]color_write_addr;
input		color_write_clock;
input		color_write_enable;

//synopsys translate_off

wire 		color_read_clock;
wire [3:0] 	color_read_data;
wire [3:0] 	color_read_data_low;
wire [3:0] 	color_read_data_high;
wire [12:0] color_read_addr;
wire 		color_read_enable;

wire 		color_write_clock;
wire [3:0] 	color_write_data;
wire [12:0] color_write_addr;
wire 		color_write_enable;
wire 		low_color_write_enable;
wire 		high_color_write_enable;
wire 		low_color_read_enable;
wire 		high_color_read_enable;

assign low_color_write_enable = color_write_enable & !color_write_addr[12];
assign high_color_write_enable = color_write_enable & color_write_addr[12];
assign low_color_read_enable = color_read_enable & !color_read_addr[12];
assign high_color_read_enable = color_read_enable & color_read_addr[12];

assign color_read_data = color_read_addr[12] ? color_read_data_high:color_read_data_low;


// instantiate the RAMs

`library("RAM_LOW","virtex2p")
`library("RAM_HIGH","virtex2p")


RAMB16_S4_S4 RAM_LOW (
.DOA(),
.ADDRA(color_write_addr[11:0]),
.CLKA(color_write_clock),
.DIA(color_write_data[3:0]),
.ENA(low_color_write_enable),
.WEA(1'b1),
.SSRA(1'b0),

.DOB(color_read_data_low[3:0]),
.ADDRB(color_read_addr[11:0]),
.CLKB(color_read_clock),
.DIB(4'b0000),
.ENB(low_color_read_enable),
.WEB(1'b0),
.SSRB(1'b0)
);

RAMB16_S4_S4 RAM_HIGH (
.DOA(),
.ADDRA(color_write_addr[11:0]),
.CLKA(color_write_clock),
.DIA(color_write_data[3:0]),
.ENA(high_color_write_enable),
.WEA(1'b1),
.SSRA(1'b0),

.DOB(color_read_data_high[3:0]),
.ADDRB(color_read_addr[11:0]),
.CLKB(color_read_clock),
.DIB(4'b0000),
.ENB(high_color_read_enable),
.WEB(1'b0),
.SSRB(1'b0)
);

//synopsys translate_on

endmodule //COLOR_RAM


module DRIVE_DAC_DATA (
bm_red_data, 
bm_green_data, 
bm_blue_data, 
char_mode_data,
dac_red_data,
dac_green_data,
dac_blue_data
) /* synthesis syn_black_box */ ;

input [7:0] 	bm_red_data;		// bit mapped red data from video memory
input [7:0] 	bm_green_data;		// bit mapped green data from video memory
input [7:0]		bm_blue_data;		// bit mapped blue data from video memory
input [7:0] 	char_mode_data;		// character mapped data from character generator
output [7:0] 	dac_red_data;		// the red data for the video dac
output [7:0] 	dac_green_data;		// the green data for the video dac
output [7:0] 	dac_blue_data;		// the blue data for the video dac
	
//synopsys translate_off

reg [7:0]		dac_red_data;
reg [7:0]		dac_green_data;
reg [7:0]		dac_blue_data;


	// make the dac connections
	always @ (bm_red_data or bm_green_data or bm_blue_data or char_mode_data) begin
		dac_red_data   = (bm_red_data & char_mode_data);
		dac_green_data = (bm_green_data & char_mode_data);
		dac_blue_data  = (bm_blue_data & char_mode_data);
	end
	
//synopsys translate_on
	
endmodule //DRIVE_DAC_DATA

`define CHARACTER_DECODE_DELAY 5


// 800 X 600 @ 60Hz with a 40.000MHz pixel clock
`define H_ACTIVE		800	// pixels
`define H_FRONT_PORCH	40	// pixels
`define H_SYNCH			128	// pixels
`define H_BACK_PORCH	88	// pixels
`define H_TOTAL			1056// pixels

`define V_ACTIVE		600	// lines
`define V_FRONT_PORCH	1	// lines
`define V_SYNCH			4	// lines
`define V_BACK_PORCH	23	// lines
`define V_TOTAL			628	// lines


module SVGA_TIMING_GENERATION (
pixel_clock,
reset,
h_synch_delay,
v_synch_delay,
comp_synch,
blank,
char_line_count,
char_address,
char_pixel
) /* synthesis syn_black_box */ ; 

input 			pixel_clock;	// pixel clock 
input 			reset;			// reset
output 			h_synch_delay;	// horizontal synch for VGA connector
output 			v_synch_delay;	// vertical synch for VGA connector
output 			comp_synch;		// composite synch for DAC
output			blank;			// composite blanking 
output [2:0]	char_line_count;// line counter for char gen rom 
output [12:0]	char_address;	// character mode address			
output [2:0]	char_pixel;		// pixel position within the character 

//synopsys translate_off

reg [9:0]		line_count;		// counts the display lines
reg [10:0]		pixel_count;	// counts the pixels in a line	
reg				h_synch;		// horizontal synch
reg				v_synch;		// vertical synch
reg				h_synch_delay;	// h_synch delayed 2 clocks to line up with DAC pipeline
reg				v_synch_delay;	// v_synch delayed 2 clocks to line up with DAC pipeline
reg				h_synch_delay0;	// h_synch delayed 1 clock
reg				v_synch_delay0;	// v_synch delayed 1 clock

reg				h_c_synch;		// horizontal component of comp synch
reg				v_c_synch;		// vertical component of comp synch
reg				comp_synch;		// composite synch for DAC
reg				h_blank;		// horizontal blanking
reg				v_blank;		// vertical blanking
reg				blank;			// composite blanking
	

reg [2:0] 		char_line_count;// identifies the line number within a character block
reg [16:0] 		char_count;		// a counter used to define the character block number
reg [16:0] 		line_start_address;// register to store the starting character number for a line of chars
reg 			reset_char_count; // flag to reset the character count during VBI
reg 			hold_char_count;// flag to hold the character count during HBI
reg [12:0] 		char_address;	// the actual character address
reg [2:0]   	char_pixel;		// pixel position within the character


// CREATE THE HORIZONTAL LINE PIXEL COUNTER
always @ (posedge pixel_clock or posedge reset) begin
	if (reset)
 		begin						// on reset set pixel counter to 0
			pixel_count <= 11'h000;
		end

	else if (pixel_count == (`H_TOTAL - 1))
 		begin							// last pixel in the line
			pixel_count <= 11'h000;		// reset pixel counter
		end

	else 	begin
			pixel_count <= pixel_count +1;		
		end
	end

// CREATE THE HORIZONTAL SYNCH PULSE
always @ (posedge pixel_clock or posedge reset) begin
	if (reset)
 		begin						// on reset
			h_synch <= 1'b0;		// remove h_synch
		end

	else if (pixel_count == (`H_ACTIVE + `H_FRONT_PORCH -1)) 
	  	begin					// start of h_synch
			h_synch <= 1'b1;
		end

	else if (pixel_count == (`H_TOTAL - `H_BACK_PORCH -1))
 	 	begin					// end of h_synch
			h_synch <= 1'b0;
		end
	end


// CREATE THE VERTICAL FRAME LINE COUNTER
always @ (posedge pixel_clock or posedge reset) begin
	if (reset)
 		begin							// on reset set line counter to 0
			line_count <= 10'h000;
		end

	else if ((line_count == (`V_TOTAL - 1))&& (pixel_count == (`H_TOTAL - 1)))
		begin							// last pixel in last line of frame 
			line_count <= 10'h000;		// reset line counter
		end

	else if ((pixel_count == (`H_TOTAL - 1)))
		begin							// last pixel but not last line
			line_count <= line_count + 1;// increment line counter
		end
	end

// CREATE THE VERTICAL SYNCH PULSE
always @ (posedge pixel_clock or posedge reset) begin
	if (reset)
 		begin							// on reset
			v_synch = 1'b0;				// remove v_synch
		end

	else if ((line_count == (`V_ACTIVE + `V_FRONT_PORCH -1) &&
		   (pixel_count == `H_TOTAL - 1))) 
	  	begin							// start of v_synch
			v_synch = 1'b1;
		end
	
	else if ((line_count == (`V_TOTAL - `V_BACK_PORCH - 1))	&&
		   (pixel_count == (`H_TOTAL - 1)))
	 	begin							// end of v_synch
			v_synch = 1'b0;
		end
	end

// ADD TWO PIPELINE DELAYS TO THE SYNCHs COMPENSATE FOR THE DAC PIPELINE DELAY
always @ (posedge pixel_clock or posedge reset) begin
	if (reset)
 		begin					
			h_synch_delay0 <= 1'b0;
			v_synch_delay0 <= 1'b0;
			h_synch_delay  <= 1'b0;
			v_synch_delay  <= 1'b0;

		end
	else 
		begin
			h_synch_delay0 <= h_synch;
			v_synch_delay0 <= v_synch;
			h_synch_delay  <= h_synch_delay0;
			v_synch_delay  <= v_synch_delay0;
		end
	end



// CREATE THE HORIZONTAL BLANKING SIGNAL
// the "-2" is used instead of "-1" because of the extra register delay
// for the composite blanking signal 
always @ (posedge pixel_clock or posedge reset) begin
	if (reset)
 		begin					// on reset
			h_blank <= 1'b0;	// remove the h_blank
		end

	else if (pixel_count == (`H_ACTIVE -2)) 
	  	begin					// start of HBI
			h_blank <= 1'b1;
		end
	
	else if (pixel_count == (`H_TOTAL -2))
 	 	begin					// end of HBI
			h_blank <= 1'b0;
		end
	end


// CREATE THE VERTICAL BLANKING SIGNAL
// the "-2" is used instead of "-1"  in the horizontal factor because of the extra
// register delay for the composite blanking signal 
always @ (posedge pixel_clock or posedge reset) begin
	if (reset)
 		begin						// on reset
			v_blank <= 1'b0;		// remove v_blank
		end

	else if ((line_count == (`V_ACTIVE - 1) &&
		   (pixel_count == `H_TOTAL - 2))) 
	  	begin						// start of VBI
			v_blank <= 1'b1;
		end
	
	else if ((line_count == (`V_TOTAL - 1)) &&
		   (pixel_count == (`H_TOTAL - 2)))
	 	begin						// end of VBI
			v_blank <= 1'b0;
		end
	end


// CREATE THE COMPOSITE BANKING SIGNAL
always @ (posedge pixel_clock or posedge reset) begin
	if (reset)
		begin						// on reset
			blank <= 1'b0;			// remove blank
		end

	else if (h_blank || v_blank)	// blank during HBI or VBI
		 begin
			blank <= 1'b1;
		end
	else begin
			blank <= 1'b0;			// active video do not blank
		end
	end
		
// CREATE THE HORIZONTAL COMPONENT OF COMP SYNCH
// the "-2" is used instead of "-1" because of the extra register delay
// for the composite synch
always @ (posedge pixel_clock or posedge reset) begin
	if (reset)
 		begin					// on reset
			h_c_synch <= 1'b0;	// remove h_c_synch
		end

	else if (pixel_count == (`H_ACTIVE + `H_FRONT_PORCH -2)) 
	  	begin					// start of h_c_synch
			h_c_synch <= 1'b1;
		end


	else if (pixel_count == (`H_TOTAL - `H_BACK_PORCH -2))
 	 	begin					// end of h_c_synch
			h_c_synch <= 1'b0;
		end
	end

// CREATE THE VERTICAL COMPONENT OF COMP SYNCH 
always @ (posedge pixel_clock or posedge reset) begin
	if (reset)
 		begin						// on reset
			v_c_synch <= 1'b0;		// remove v_c_synch
		end

	else if ((line_count == (`V_ACTIVE + `V_FRONT_PORCH - 1) &&
		   (pixel_count == `H_TOTAL - 2))) 
	  	begin						// start of v_c_synch
			v_c_synch <= 1'b1;
		end
	
	else if ((line_count == (`V_TOTAL - `V_BACK_PORCH - 1))	&&
		   (pixel_count == (`H_TOTAL - 2)))
	 	begin						// end of v_c_synch
			v_c_synch <= 1'b0;
		end
	end

// CREATE THE COMPOSITE SYNCH SIGNAL
always @ (posedge pixel_clock or posedge reset) begin
	if (reset)
 		begin						// on reset
			comp_synch <= 1'b0;		// remove comp_synch
		end
	else begin
		comp_synch <= (v_c_synch ^ h_c_synch);
		end
	end


/* 
   CREATE THE CHARACTER COUNTER.
   CHARACTERS ARE DEFINED WITHIN AN 8 x 8 PIXEL BLOCK.

	A 640  x 480 video mode will display 80  characters on 60 lines.
	A 800  x 600 video mode will display 100 characters on 75 lines.
	A 1024 x 768 video mode will display 128 characters on 96 lines.

	"char_line_count" identifies the row in the 8 x 8 block.
	"char_pixel" identifies the column in the 8 x 8 block.
	"char_address" identifies the character in the total diaplay "0"
	is the top left character.
*/


// CREATE THE VERTICAL FRAME LINE COUNTER
always @ (posedge pixel_clock or posedge reset) begin
	if (reset)
 		begin							// on reset set line counter to 0
			char_line_count <= 3'b000;
		end

	else if  ((line_count == (`V_TOTAL - 1)) && (pixel_count == (`H_TOTAL - 1)-`CHARACTER_DECODE_DELAY)) 
		begin							
			char_line_count <= 3'b000;		// reset line counter
		end

	else if (pixel_count == (`H_TOTAL - 1)-`CHARACTER_DECODE_DELAY)
		begin							
			char_line_count <= line_count + 1;// increment line counter
		end
	end

always @ (posedge pixel_clock or posedge reset) begin
	if (reset)
		begin
			char_count <= 17'h00000;
			line_start_address <= 17'h00000;
		end
	else if (reset_char_count)		// reset the char count during the VBI
		begin
			char_count <= 17'h00000;
			line_start_address <= 17'h00000;
		end
	else if (!hold_char_count)
			begin
				char_count <= char_count +1;
				line_start_address <= line_start_address;
			end
	else 		// hold the character counter during the HBI
 		begin
			if (char_line_count == 3'b111)				// last line in the character block 
				begin
					char_count <= char_count;
					line_start_address <= char_count; 	// update the line start address
				end
			else										// not the last line in the char block
				begin
					char_count <= line_start_address;	// restore the line start address
					line_start_address <= line_start_address;
				end
			end
		end
	
always @ (char_count) begin
	char_address[12:0] = char_count[15:3];
	end

// char_pixel defines the pixel within the character line
always @ (posedge pixel_clock or posedge reset) begin
	if (reset) begin
		char_pixel <= 3'b101;		// reset to 5 so that the first character data can be latched
		end
	else if (pixel_count == ((`H_TOTAL - 1)-`CHARACTER_DECODE_DELAY))begin
		char_pixel <= 3'b101;		// reset to 5 so that the first character data can be latched
		end
	else begin
		char_pixel <= char_pixel +1;
		end
	end


// CREATE THE CONTROL SIGNALS FOR THE CHARACTER ADDRESS COUNTER
/* 	The HOLD and RESET signals are advanced from the beginning and end
	of HBI and VBI to compensate for the internal character generation
	pipeline.
*/
always @ (posedge pixel_clock or posedge reset) begin
	if (reset)
		begin
			reset_char_count <= 1'b0;
		end
	else if (line_count == (`V_ACTIVE - 1) &&
		   pixel_count == ((`H_ACTIVE - 1) - `CHARACTER_DECODE_DELAY))
	  	begin								// start of VBI
				reset_char_count <= 1'b1;
		end
	
	else if (line_count == (`V_TOTAL - 1) &&
		   pixel_count == ((`H_TOTAL - 1) - `CHARACTER_DECODE_DELAY))
	 	begin								// end of VBI					
			reset_char_count <= 1'b0;
		end
	end


always @ (posedge pixel_clock or posedge reset) begin
	if (reset)
		begin
			hold_char_count <= 1'b0;
		end
	else if (pixel_count == ((`H_ACTIVE -1) - `CHARACTER_DECODE_DELAY)) 
	  	begin								// start of HBI
 			hold_char_count <= 1'b1;
		end
 	else if (pixel_count == ((`H_TOTAL -1) - `CHARACTER_DECODE_DELAY ))
	 	begin								// end of HBI
			hold_char_count <= 1'b0;
		end
	end

//synopsys translate_on

endmodule //SVGA_TIMING_GENERATION










/*
   Description: This module creates a 2k x 8 ROM that is used for video character generation.
			The ROM contents are also defined in the CHAR_GEN_ROM.ucf file.

			The charcter generator defines a 8 x 8 block of pixels used for ASCII characters,
			line graphics and Xilinx logo. The least significant 3 address bits define the line
			within the character block and the most significant 8 bits define the specific character.
			
			If the user wishes to extend the character set then the "INIT_20 - INIT_3F" values must be
			defined.

Code 	  00h  	defines a solid block
Codes 05h-1Fh	defines line graphics		
Codes 20h-7Fh 	defines the ASCII characters
Codes 80h-FFh  	user defined characters
*/


module CHAR_GEN_ROM (pixel_clock,address,data) /* synthesis syn_black_box */ ;
input			pixel_clock;
input [10:0]	address;
output [7:0] 	data;

//synopsys translate_off

wire           	pixel_clock;
wire  [10:0]   	address;
wire   [7:0]   	data;
 
`library("CHARACTER_GEN_ROM","virtex2p")


RAMB16_S9 
CHARACTER_GEN_ROM (
.DO(data),
.DOP(),
.ADDR(address[10:0]),
.DI(8'h00),
.DIP(1'b0),
.EN(1'b1),
.CLK(pixel_clock),
.WE(1'b0),				// no write enable
.SSR(1'b0)
)/*synthesis xc_props="INIT_01 = FF8080808080808001010101010101FF80808080808080FF0000000000000000,\
INIT_02 = 01010101FF01010110101010101010FF10101010FF101010FF01010101010101,\
INIT_03 = 00102040FF40201000080402FF02040880808080FF808080FF10101010101010,\
INIT_04 = 00000000FF0000000000000000FF0000000000000000FF0000000000000000FF,\
INIT_05 = FF0000000000000000FF0000000000000000FF0000000000000000FF00000000,\
INIT_06 = 1010101010101010202020202020202040404040404040408080808080808080,\
INIT_07 = 0101010101010101020202020202020204040404040404040808080808080808,\
INIT_08 = 0028287C287C2828000000000028282800100000101010100000000000000000,\
INIT_09 = 0000000000201030003448542050483000064620100864600010781438503C10,\
INIT_0A = 000010107C101000000010543854100000201008080810200008102020201008,\
INIT_0B = 00004020100804000030300000000000000000007C0000002010300000000000,\
INIT_0C = 003844040810087C007C201008044438003810101010301000384464544C4438,\
INIT_0D = 002020201008047C0038444478402018003844040478407C0008087C48281808,\
INIT_0E = 20103000003030000030300000303000003008043C4444380038444438444438,\
INIT_0F = 001000100804443800201008040810200000007C007C00000008102040201008,\                                                                           
INIT_10 = 003844404040443800784444784444780044447C444444380038545434044438,\
INIT_11 = 003C44445C404438004040407840407C007C40407840407C0070484444444870,\
INIT_12 = 0044485060504844003048080808081C0038101010101038004444447C444444,\
INIT_13 = 00384444444444380044444C546444440044444454546C44007C404040404040,\
INIT_14 = 007804043840403C004448507844447800344854444444380040404078444478,\
INIT_15 = 002854545444444400102844444444440038444444444444001010101010107C,\
INIT_16 = 0038202020202038007C40201008047C00101010284444440044442810284444,\
INIT_17 = 007C000000000000000000000044281000380808080808380000040810204000,\
INIT_18 = 00384440403800000078444464584040003C443C043800000000000000081020,\
INIT_19 = 38043C44443C000000202020702024180038407C44380000003C44444C340404,\
INIT_1A = 0048506050484040304808081800080000381010103010100044444464584040,\
INIT_1B = 0038444444380000004444446458000000444454546800000038101010101030,\
INIT_1C = 0078043840380000004040406458000004043C4C340000004040407844780000,\
INIT_1D = 0028545444440000001028444444000000344C44444400001824202070202000,\
INIT_1E = 0008101020101008007C2010087C000038043C44440000000044281028440000,\
INIT_1F = AA55AA55AA55AA550000000C9260000000201010081010200010101010101010"*/;


//synopsys translate_on

endmodule//CHAR_GEN_ROM